// -*- mode: c++; -*-
// Copyright (C) 2024
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef AUTOMATON_H
#define AUTOMATON_H

#include <cstdint>
#include <iostream>

// -- Namespaces: ---------------------------------------------------------
namespace P4 {
  // -- Type aliases: -------------------------------------------------------

  // -- Classes: ------------------------------------------------------------

  /// Clase base que representa a un autómata general.
  class Automaton {
  public:
    // -- Constructors/Destructor: --------------------------------------------
    /// Constructor de un autómata a partir de un caracter c
    Automaton(char c);
    /// Esta clase no tiene constructor por defecto
    Automaton() = delete;
    /// Implementación por defecto por si más adelante lo necesitamos redefinir
    virtual ~Automaton() = default;

    // -- Setters: ------------------------------------------------------------
    /// Cambia el carácter de dibujo de este autómata
    void set_drawing_char(char c);

    // -- Getters: ------------------------------------------------------------
    /// Devuelve el carácter de dibujo de este autómata
    char get_drawing_char() const;

    // -- Action: -------------------------------------------------------------
    /// Método abstracto en esta clase, sirve para evolucionar el
    /// autómata durante ngens generaciones.
    virtual void compute_ngens(uint32_t ngens, std::ostream& os) = 0;

  protected:
    char c; ///< El carácter usado para dibujar este autómata

  };

}  // P4

#endif /* AUTOMATON_H */
