// -*- mode: c++; -*-
// Copyright (C) 2024
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CACONTAINER_H
#define CACONTAINER_H

#include "ca.h"
#include "container.tcc"

namespace P4 {

  /// Clase que representa a los contenedores de autómatas celulares
  /// 1-dimensionales
  class CaContainer : public Container<CellularAutomatonPtr>
  {
  public:
    // -- Constructors/Destructor: --------------------------------------------
    /// Crea un CaContainer con nombre name
    CaContainer(const std::string &name);
    /// Implementación por defecto por si más adelante lo necesitamos redefinir
    virtual ~CaContainer() = default;

    // -- Action: -------------------------------------------------------------

    /// Devuelve en una cadena la generación actual de cada autómata
    /// celular que contiene encerrada entre "[]". La cadena
    /// correspondiente a cada autómata se separa de la siguiente por un
    /// carácter '\n', p.e. el siguiente contenedor de autómatas celulares
    /// contiene 3 de estos automatas, cuya generación actual es:
    /// "[-- -- ----- ---]
    ///  [--- -- ---   --]
    ///  [-- -- -  -- - -]"
    std::string get_all_cas_current_gen();

  };

}  // P4

#endif /* CACONTAINER_H */
