// Copyright (C) 2024
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <cstdint>
#include <iostream>
#include <cstdlib>
#include "ca.h"
#include "cacontainer.h"

// Consulta : https://es.wikipedia.org/wiki/Aut%C3%B3mata_celular_elemental

using namespace std;

void f([[maybe_unused]] uint32_t g, [[maybe_unused]] const P4::Cells_t &cells,
       [[maybe_unused]] const P4::CellularAutomaton &ca) {

  cout << "\nSignal nxgen_sig caught:\nGen: "
       << g << "\nCA drawing char:" << ca.get_drawing_char() << endl;
}

void namechange_fn(const string &oldn, const string &newn) {
  cout << "Container name change from: "
       << oldn << " to: " << newn << endl;
}

void usage() {
  cout << "Usage: p4 --help|ruleset [ncells] [ngens]\n";
}

int main([[maybe_unused]] int argc, [[maybe_unused]] char *argv[]) {
  uint64_t ncells = 40;
  uint8_t rset = 30;
  uint32_t ngens = 20;
  string help = "--help";

  P4::CaContainer cac("ca-Container-1");
  cac.namechange_sig.connect(namechange_fn);

  // manage program arguments
  switch (argc) {
  case 2: // argv[1] == rset || --help
    if (help == argv[1]) {
      usage();
      return 0;
    } else {
      rset = static_cast<uint8_t>(std::stoi(argv[1]));
    }
    break;
  case 3: // argv[1] == rset && argv[2] == ncells
    if (help == argv[1]) {
      usage();
      return 0;
    } else {
      ncells = std::stoul(argv[2]);
      rset = static_cast<uint8_t>(std::stoi(argv[1]));
    }
    break;
  case 4: // argv[1] == rset && argv[2] == ncells && argv[3] == ngens
    if (help == argv[1]) {
      usage();
      return 0;
    } else {
      ncells = std::stoul(argv[2]);
      rset = static_cast<uint8_t>(std::stoi(argv[1]));
      ngens = static_cast<uint32_t>(std::stoul(argv[3]));
    }
    break;
  default:
    break;
  }

  P4::CellularAutomaton ca(ncells, '.', rset);
  P4::CellularAutomaton ca2(ncells, '@', rset+3);
  cac.add(&ca);
  cac.add(&ca2);
  cout << "Container " << cac.get_name()
  << " has " << cac.size() << " elements.\n";
  //ca.nxgen_sig.connect(f);

  cout << "\nca:\n" << ca << "\n";
  cout << "\nca2:\n" << ca2 << "\n";

  cout << "Current gen. of all cas in cac:\n"
  << cac.get_all_cas_current_gen() << endl;

  // ca.print_current_gen(cout);
  // cout << endl;
  ca.compute_ngens(ngens, cout);

  cac.set_name("NUEVO NOMBRE");

  cout << "\nCurrent gen. of all cas in cac:\n"
  << cac.get_all_cas_current_gen() << endl;

  return 0;
}
